﻿using System.ComponentModel;
using System.Windows.Input;
using Xamarin.Forms;

namespace GaugeTestApp
{
    public class MainViewModel : INotifyPropertyChanged
    {
        private const int HeightMin = 100;
        private const int HeightMax = 400;
        private const int HeightInc = 100;

        public SemiCircleGauge Gauge1 { get; private set; }

        private double _gaugeHeight;
        public double GaugeHeight
        {
            get { return _gaugeHeight; }
            set
            {
                _gaugeHeight = value;
                OnPropertyChanged("GaugeHeight");
            }
        }

        public ICommand DecreaseSizeCommand { get; set; }
        public ICommand IncreaseSizeCommand { get; set; }

        public MainViewModel()
        {
            Gauge1 = new SemiCircleGauge();
            Gauge1.SetMinMax(0, 100);
            Gauge1.SetIncrement(10);
            Gauge1.SetLine_StartEnd(0, 75, LineNo.Line1);
            Gauge1.SetLine_StartEnd(80, 100, LineNo.Line2);
            Gauge1.SetValue(50);

            GaugeHeight = HeightMax;

            DecreaseSizeCommand = new Command(() =>
            {
                if (GaugeHeight > HeightMin)
                    GaugeHeight -= HeightInc;
            });

            IncreaseSizeCommand = new Command(() =>
            {
                if (GaugeHeight < HeightMax)
                    GaugeHeight += HeightInc;
            });
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
