﻿using Steema.TeeChart;
using Steema.TeeChart.Styles;
using System;
using System.IO;
using Xamarin.Forms;

namespace GaugeTestApp
{
    public class SemiCircleGauge : ChartView
    {
        #region Properties

        public CircularGauge CircularGauge { get; private set; }

        #endregion

        #region ctor

        public SemiCircleGauge()
        {
            CreateGauge();
        }

        private void CreateGauge()
        {
            ChartView cv = this;

            cv.VerticalOptions = LayoutOptions.FillAndExpand;
            cv.HorizontalOptions = LayoutOptions.FillAndExpand;

            cv.Chart.Aspect.View3D = false;

            // ### CHART LEGEND ###
            cv.Chart.Legend.Visible = false;

            // ### CHART HEADER ###
            cv.Chart.Header.Text = "";
            cv.Chart.Header.Visible = false;
            cv.Chart.Header.Height = 0;
            cv.Chart.Header.Alignment = TextAlignment.Center;

            // ### CHART FOOTER ###
            cv.Chart.Footer.Text = "";
            cv.Chart.Footer.Visible = false;
            cv.Chart.Footer.Height = 0;
            cv.Chart.Footer.Alignment = TextAlignment.Center;

            // ### PANEL ###
            cv.Chart.Panel.Gradient.Visible = false;
            cv.Chart.Panel.Color = Color.Transparent;
            cv.Chart.Panel.MarginUnits = PanelMarginUnits.Percent;
            cv.Chart.Panel.MarginTop = 2;
            cv.Chart.Panel.MarginBottom = 2;
            cv.Chart.Panel.MarginRight = 2;
            cv.Chart.Panel.MarginLeft = 2;

            // ### CIRCULAR GAUGE ###
            CircularGauge = new CircularGauge();
            cv.Chart.Series.Add(CircularGauge);
            CircularGauge.Title = "";

            // ### TICKS ###
            CircularGauge.Ticks.Visible = true;
            CircularGauge.Ticks.Color = Color.White;
            CircularGauge.MinorTicks.Visible = true;
            CircularGauge.MinorTickDistance = 2;
            CircularGauge.MinorTicks.Color = Color.White;
            CircularGauge.MinorTicks.VertSize = 4;
            CircularGauge.MinorTicks.HorizSize = 3;
            CircularGauge.MinorTicks.Pen.Visible = false;

            // ### HANDS ###
            CircularGauge.HandDistance = 50;
            CircularGauge.HandOffset = 0;
            CircularGauge.Hand.HorizSize = 8;
            CircularGauge.Hand.Color = Color.FromRgb(220, 220, 220);
            CircularGauge.Hand.Draw3D = false;
            CircularGauge.Hand.Pen.Visible = false;
            CircularGauge.Hand.Gradient.Visible = false;
            CircularGauge.Hand.Shadow.Visible = true;
            CircularGauge.Hand.Style = PointerStyles.DownTriangle;

            // ### CENTER ###
            CircularGauge.Center.Gradient.Visible = true;
            CircularGauge.Center.Gradient.StartColor = Color.FromRgb(51, 51, 51);
            CircularGauge.Center.Gradient.EndColor = Color.FromRgb(0, 0, 0);
            CircularGauge.Center.HorizSize = 20;
            CircularGauge.Center.VertSize = 20;
            CircularGauge.Center.Draw3D = false;
            CircularGauge.Center.Pen.Visible = true;
            CircularGauge.Center.Pen.Color = Color.FromRgb(198, 198, 198);
            CircularGauge.Center.Pen.Width = 1;
            CircularGauge.Center.Shadow.Visible = true;

            // ### FACE ###
            CircularGauge.FaceBrush.Visible = true;
            CircularGauge.FaceBrush.Solid = true;
            CircularGauge.FaceBrush.Color = Color.Transparent;
            CircularGauge.FaceBrush.Gradient.Visible = false;

            // ### GREEN LINE (LINE 1) ###
            CircularGauge.GreenLine.Visible = true;
            CircularGauge.GreenLine.Gradient.Visible = true;
            CircularGauge.GreenLine.Gradient.UseMiddle = true;
            CircularGauge.GreenLine.Gradient.StartColor = Color.LightGreen;
            CircularGauge.GreenLine.Gradient.EndColor = Color.Red;
            CircularGauge.GreenLine.Gradient.Direction = Steema.TeeChart.Drawing.GradientDirection.DiagonalUp;
            CircularGauge.GreenLine.Position = 50;
            CircularGauge.GreenLine.Spiralled = false;
            CircularGauge.GreenLine.InflateMargins = true;
            CircularGauge.GreenLine.HorizSize = 8;

            // ### RED LINE (LINE 2) ###
            CircularGauge.RedLine.Visible = true;
            CircularGauge.RedLine.Gradient.Visible = true;
            CircularGauge.RedLine.Gradient.UseMiddle = true;
            CircularGauge.RedLine.Gradient.StartColor = Color.Orange;
            CircularGauge.RedLine.Gradient.EndColor = Color.Orange;
            CircularGauge.RedLine.Gradient.Direction = Steema.TeeChart.Drawing.GradientDirection.DiagonalUp;
            CircularGauge.RedLine.Position = 50;
            CircularGauge.RedLine.Spiralled = false;
            CircularGauge.RedLine.InflateMargins = true;
            CircularGauge.RedLine.HorizSize = 8;

            // ### FRAME ###
            CircularGauge.Frame.Visible = true;
            CircularGauge.Frame.FrameElementPercents = new double[3] { 30, 30, 40 }; // Outer, Inner, Middle
            CircularGauge.Frame.Circled = true;
            CircularGauge.Frame.Width = 5;
            CircularGauge.Frame.RotationAngle = 0;
            CircularGauge.Frame.TotalAngle = 360;
            // Inner Band
            CircularGauge.Frame.InnerBand.Visible = true;
            CircularGauge.Frame.InnerBand.Solid = true;
            CircularGauge.Frame.InnerBand.Gradient.Visible = false;
            CircularGauge.Frame.InnerBand.Color = Color.White;
            // Middle Band
            CircularGauge.Frame.MiddleBand.Visible = true;
            CircularGauge.Frame.MiddleBand.Solid = true;
            CircularGauge.Frame.MiddleBand.Gradient.Visible = false;
            CircularGauge.Frame.MiddleBand.Color = Color.Transparent;
            // Outer Band
            CircularGauge.Frame.OuterBand.Visible = true;
            CircularGauge.Frame.OuterBand.Solid = true;
            CircularGauge.Frame.OuterBand.Gradient.Visible = false;
            CircularGauge.Frame.OuterBand.Color = Color.White;

            // ### AXIS ###
            CircularGauge.RotateLabels = false;
            CircularGauge.Axis.Labels.Visible = true;
            CircularGauge.Axis.Labels.Font.Color = Color.White;
            CircularGauge.Axis.MinorTickCount = 4;
            CircularGauge.Axis.Labels.AutoSize = true;
            CircularGauge.Axis.Labels.Font.Size = 7;

            CircularGauge.Axis.SetMinMax(0.0D, 1.0D);

            SetIncrement(0.5);

            // ### VALUE INFO ###
            SetMinMax(0.0D, 1.0D);
            CircularGauge.Value = 0.0;

            // ### ANGLES ###
            CircularGauge.DisplayTotalAngle = 360;
            CircularGauge.DisplayRotationAngle = 0;
            CircularGauge.TotalAngle = 180;

            cv.Chart.AfterDraw += Chart_AfterDraw;
        }

        #endregion

        #region Draw

        private void Chart_AfterDraw(object sender, Steema.TeeChart.Drawing.Graphics3D g)
        {
            DrawBattery(g);
            Resize();
        }

        private void DrawBattery(Steema.TeeChart.Drawing.Graphics3D g)
        {
            double imgX = CircularGauge.YRadius / 1.5; //  2;
            double imgY = imgX;

            var xpos = g.ChartXCenter - (imgX / 2);
            var ypos = g.ChartYCenter + ((CircularGauge.YRadius / 2) - (imgY / 2));

            g.Brush.Visible = false;
            g.Pen.Width = 4; // 9; //could scale this according to dimensions

            //relative dimensions
            double left = xpos + (imgX * 0.13);
            double top = ypos + (imgY * 0.27);
            double right = xpos + imgX - (imgX * 0.13);
            double bottom = ypos + imgY - (imgY * 0.24);
            double width = right - left;
            double height = bottom - top;
            double sigBar = width * 3 / 16; //define size for symbols

            //top connectors
            g.Pen.Color = Xamarin.Forms.Color.FromRgb(222, 222, 222);
            g.Line(new Point(left + (width / 4) - (sigBar / 3), top - 4), new Point(left + (width / 4) + (sigBar / 3), top - 4));
            g.Line(new Point(right - (width / 4) - (sigBar / 3), top - 4), new Point(right - (width / 4) + (sigBar / 3), top - 4));

            //battery
            g.Pen.Color = Xamarin.Forms.Color.FromRgb(201, 201, 201);
            g.Rectangle(left, top, right, bottom);

            //plus
            g.Pen.Color = Xamarin.Forms.Color.FromRgb(218, 218, 218);
            g.Line(new Point(left + (width / 4) - (sigBar / 2), top + (height * 3 / 8)), new Point(left + (width / 4) + (sigBar / 2), top + (height * 3 / 8)));
            g.Line(new Point(left + (width / 4), top + (height * 3 / 8) - (sigBar / 2)), new Point(left + (width / 4), top + (height * 3 / 8) + (sigBar / 2)));

            //minus
            g.Line(new Point(right - (width / 4) - (sigBar / 2), top + (height * 3 / 8)), new Point(right - (width / 4) + (sigBar / 2), top + (height * 3 / 8)));
        }

        public void Resize()
        {
            var linePos = 50;
            var lineWidth = CircularGauge.XRadius * 0.05D;
            var centerSize = CircularGauge.XRadius * 0.1D;

            System.Diagnostics.Debug.WriteLine($"CircularGauge.XRadius = {CircularGauge.XRadius}");
            System.Diagnostics.Debug.WriteLine($"linePos = {linePos}");
            System.Diagnostics.Debug.WriteLine($"lineWidth = {lineWidth}");
            System.Diagnostics.Debug.WriteLine($"centerSize = {centerSize}");

            try
            {
                // ### CENTER ###
                CircularGauge.Center.HorizSize = centerSize;
                CircularGauge.Center.VertSize = centerSize;

                // ### HANDS ###
                CircularGauge.HandDistance = linePos;
                CircularGauge.Hand.HorizSize = lineWidth;

                // ### GREEN LINE (LINE 1) ###
                CircularGauge.GreenLine.Position = linePos;
                CircularGauge.GreenLine.HorizSize = lineWidth;

                // ### RED LINE (LINE 2) ###
                CircularGauge.RedLine.Position = linePos;
                CircularGauge.RedLine.HorizSize = lineWidth;

                // ### AXIS ###
                CircularGauge.Axis.Labels.Font.Size = 7.0D * (CircularGauge.XRadius / 100D);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.ToString());
            }
        }

        #endregion

        #region Public Methods

        public void SetMinMax(double min, double max, LineNo lineNo = LineNo.Line1)
        {
            CircularGauge.Minimum = min;
            CircularGauge.Maximum = max;
            if (CircularGauge.Value < min)
                CircularGauge.Value = min;
            else if (CircularGauge.Value > max)
                CircularGauge.Value = max;
        }

        public void SetLine_StartEnd(double start, double end, LineNo lineNo = LineNo.Line1)
        {
            switch (lineNo)
            {
                case LineNo.Line2:
                    CircularGauge.RedLineStartValue = start;
                    CircularGauge.RedLineEndValue = end;
                    break;

                default:
                    CircularGauge.GreenLineStartValue = start;
                    CircularGauge.GreenLineEndValue = end;
                    break;
            }
        }

        public void SetIncrement(double inc)
        {
            CircularGauge.Axis.Increment = inc;
            CircularGauge.Axis.SetMinMax(0.0D, 1.0D);
        }

        public void SetValue(double newValue) => CircularGauge.Value = newValue;

        #endregion

    }

    public enum LineNo
    {
        Line1,
        Line2
    }
}